<style>
    div#maintoolbar {
     display: none !important;
    }
    .notebook_app {
     background-color: #ffffff;
    }
    #notebook-container {
     padding: 0px;
     background-color: #ffffff;
     min-height: 0px;
     box-shadow: none;
    }
    .container {
     width: 900px;
    }
    .toolbar_info {
     color: #2c2c2c;
     font-size: 12pt;
     border: none;
     font-family: 'Open-Sans', sans-serif;
     text-shadow: none !important;
    }
    .list_container {
     color: #2c2c2c;
     font-size: 12pt;
     border: none;
     font-family: 'Open-Sans', sans-serif;
     text-shadow: none !important;
    }
    .list_container > div {
     border-bottom: 1px solid rgba(180,180,180,.25);
     font-size: 12pt;
    }
    .list_container a {
     color: #2c2c2c;
     font-size: 12pt;
     border: none;
     font-family: 'Open-Sans', sans-serif;
     text-shadow: none !important;
    }
    div.list_container a:hover {
     color: #222222;
    }
    .list_header {
     font-weight: normal;
     background-color: #ffffff;
     color: #2c2c2c;
    }
    div#notebook_list_header.row.list_header {
     background-color: #ffffff;
    }
    ul#selector-menu.dropdown-menu > li > a {
     padding: none;
    }
    div.list_item:hover {
     background-color: rgba(150,150,150,.10);
    }
    .breadcrumb > li {
     color: #2c2c2c;
     font-size: 12pt;
     border: none;
     font-family: 'Open-Sans', sans-serif;
     text-shadow: none !important;
    }
    .nav-tabs > li > a:hover {
     color: #222222;
     background: #ffffff;
     background-color: rgba(150,150,150,.10);
     border-color: transparent;
     background-image: none;
    }
    .nav-tabs > li.active > a,
    .nav-tabs > li.active > a:hover,
    .nav-tabs > li.active > a:focus {
     color: #000000;
     background: #ffffff;
     border-color: transparent;
     background-color: rgba(180,180,180,.25);
    }
    .nav-tabs {
     border-bottom: 1px solid rgba(150,150,150,.10);
    }
    .modal-content {
     position: relative;
     background-color: #efefef;
     border: 1px solid rgba(180,180,180,.25);
     border-radius: 3px;
     box-shadow: 0px 3px 9px rgba(180,180,180,.25);
     background-clip: padding-box;
     outline: 0px none;
    }
    .modal-header {
     padding: 15px;
     border-bottom: 1px solid rgba(180,180,180,.30);
     min-height: 16.4286px;
    }
    .panel-default {
     border-color: rgba(180,180,180,.25);
    }
    .panel {
     line-height: 22px;
     fontsize: 12px;
     margin-bottom: 18px;
     background-color: #ffffff;
     color: #2c2c2c;
     border: 0px;
     box-shadow: none;
    }
    #running .panel-group .panel .panel-heading {
     background-color: #ffffff;
     padding: 4px 7px;
     line-height: 22px;
    }
    #running .panel-group .panel .panel-heading a:hover {
     color: #ff8132;
    }
    #running .panel-group .panel .panel-body .list_container .list_item {
     border-bottom: 1px solid rgba(180,180,180,.30);
    }
    .form-control {
     display: block;
     width: 100%;
     height: 32px;
     padding: 6px 12px;
     font-size: 13pt;
     line-height: 1.6;
     color: #2c2c2c;
     background-color: #eeeeee;
     background-image: none;
     border: 1px solid rgba(180,180,180,.25);
     border-radius: 2px;
     box-shadow: none;
     transition: border-color 0.15s ease-in-out 0s, box-shadow 0.15s ease-in-out 0s;
    }
    .list_item input:not([type="checkbox"]) {
     margin-right: 0px;
     padding-right: 0px;
     height: 80%;
     width: 30%;
    }
    .modal-footer {
     padding: 15px;
     text-align: right;
     border-top: 1px solid rgba(180,180,180,.30);
    }
    .input-form-control:hover:active {
     background-color: rgba(150,150,150,.10);
     selected: rgba(150,150,150,.10);
    }
    .item_icon {
     color: #1e70c7;
    }
    .item_buttons .kernel_name {
     color: #1e70c7;
    }
    .running_notebook_icon:before {
     color: #ff8132 !important;
    }
    .item_buttons .running-indicator {
     padding-top: 4px;
     color: #ff8132;
    }
    #modal_indicator {
     float: right !important;
     color: #ff8132;
    }
    #kernel_indicator {
     float: right !important;
     color: #ff8132;
    }
    #kernel_indicator .kernel_indicator_name {
     padding-left: 5px;
     padding-right: 5px;
     font-size: 13pt;
    }
    #project-name > .breadcrumb {
     font-weight: normal;
     color: #ff8132;
    }
    div#ipython_notebook {
     display: none;
    }
    .dropdown-menu {
     border: 2px solid rgba(180,180,180,.25);
     background-color: #efefef;
     background: #efefef;
     box-shadow: none;
     padding: 0px 0;
     font-size: 12pt;
     text-align: left;
     vertical-align: middle;
    }
    .dropdown-menu:hover {
     border: 2px solid rgba(180,180,180,.25);
     background-color: #efefef;
     box-shadow: none;
     vertical-align: middle;
    }
    .dropdown-header {
     font-size: 13pt;
     display: block;
     line-height: 1.25;
     color: #2c2c2c;
     text-decoration: underline;
     text-decoration-color: rgba(180,180,180,.25);
     white-space: nowrap;
    }
    .dropdown-menu > li > a {
     font-size: 12pt;
     display: block;
     clear: both;
     padding: 8px 20px;
     font-weight: normal;
     line-height: 1.25;
     color: #2c2c2c;
     white-space: nowrap;
     background-color: #efefef;
     background: #efefef;
    }
    .dropdown-menu > li > a:hover {
     font-size: 12pt;
     color: #222222;
     display: block;
     clear: both;
     padding: 8px 20px;
     font-weight: normal;
     line-height: 1.25;
     white-space: nowrap;
     background-color: rgba(180,180,180,.25);
     background: rgba(180,180,180,.25);
    }
    .dropdown-menu .divider {
     height: 2px;
     margin: 0px 0px;
     overflow: hidden;
     background-color: rgba(180,180,180,.30);
    }
    .dropdown-submenu > .dropdown-menu {
     top: 0;
     left: 100%;
     margin-top: -2px;
     margin-left: 0px;
     padding-top: 0px;
    }
    .dropdown-menu > .disabled > a,
    .dropdown-menu > .disabled > a:hover,
    .dropdown-menu > .disabled > a:focus {
     font-weight: normal;
     color: #aaaaaa;
     padding: none;
     display: block;
     clear: both;
     line-height: 1.5;
     white-space: nowrap;
    }
    .notebook_app #header {
     box-shadow: none !important;
     background-color: #ffffff;
    }
    #header {
     box-shadow: none;
     background-color: #ffffff;
     position: relative;
     z-index: 100;
    }
    #header-container {
     padding-bottom: 5px;
     display: none !important;
     box-shadow: none;
    }
    #header .header-bar {
     background: #ffffff;
    }
    .navbar-default {
     background-color: #ffffff;
     border-color: #ffffff;
    }
    #menubar .navbar {
     border-top: 1px;
     border-radius: 0 0 2px 2px;
     margin-bottom: 0;
     margin-top: 8px;
    }
    .navbar-default .navbar-nav > li > a {
     color: #2c2c2c;
     font-size: 14pt;
    }
    .navbar-default .navbar-nav > li > a:hover,
    .navbar-default .navbar-nav > li > a:focus {
     color: #222222;
     background-color: rgba(180,180,180,.25);
    }
    .navbar-default .navbar-nav > .open > a,
    .navbar-default .navbar-nav > .open > a:hover,
    .navbar-default .navbar-nav > .open > a:focus {
     color: #000000;
     background-color: rgba(180,180,180,.30);
     border-color: rgba(180,180,180,.30);
    }
    .btn-default.active:hover,
    .open > .dropdown-toggle.btn-default:hover,
    .btn-default:active:focus,
    .btn-default.active:focus,
    .open > .dropdown-toggle.btn-default:focus,
    .btn-default:active.focus,
    .btn-default.active.focus,
    .open > .dropdown-toggle.btn-default.focus {
     color: #000000;
     background-color: rgba(180,180,180,.30);
     border-color: rgba(180,180,180,.30);
    }
    .btn {
     color: #2c2c2c;
     background: transparent;
     background-color: transparent;
     font-size: 12pt;
     font-weight: normal;
     border: 0 none !important;
     border-radius: 2px;
     box-shadow: none !important;
     text-shadow: none;
    }
    .btn-group > .btn-mini {
     color: #2c2c2c;
     background: transparent;
     background-color: transparent;
     font-size: 11pt;
     font-weight: normal;
     border: 0 none !important;
     border-radius: 2px;
     box-shadow: none !important;
     text-shadow: none;
    }
    .btn:hover {
     color: #222222;
     background: transparent;
     background-color: rgba(180,180,180,.30);
     background-image: none;
    }
    .btn-group > .btn-mini:hover {
     color: #222222;
     background: transparent;
     background-color: rgba(180,180,180,.30);
     background-image: none;
    }
    .btn-xs,
    .btn-group-xs > .btn {
     display: table-row-group;
    }
    .btn-group,
    .btn-group-vertical {
     background: transparent;
     background-color: transparent;
    }
    .alternate_upload .btn-upload {
     padding: 1px 5px;
     line-height: 1.5;
     height: 26px;
     background: transparent;
     display: inline-block;
    }
    .alternate_upload .btn-upload:hover {
     color: #222222;
     background: transparent;
     background-color: rgba(180,180,180,.30);
     background-image: none;
    }
    button.close {
     padding: 0px;
     cursor: pointer;
     background: transparent none repeat scroll 0% 0%;
     border: 0px none;
    }
    .close {
     float: right;
     font-size: 22px;
     font-weight: bold;
     line-height: 1;
     color: #ff8132;
     text-shadow: 0px 1px 0px rgba(150,150,150,.10);
     opacity: 0.3;
    }
    .close:hover {
     color: #ff8132;
     text-shadow: 0px 1px 0px rgba(150,150,150,.10);
     opacity: 0.9;
    }
    .ipython_tooltip {
     border: 2px solid rgba(180,180,180,.30);
     box-shadow: none;
     border-radius: 2px;
     position: absolute;
     z-index: 1000;
     font-size: 90%;
     background-color: #efefef;
    }
    .ipython_tooltip .tooltiptext pre {
     background-color: #efefef;
     color: #000000;
     font-size: 90%;
    }
    div#tooltip.ipython_tooltip {
     overflow: wrap;
     max-width: 825px;
    }
    div.tooltiptext.bigtooltip {
     overflow: wrap;
     height: 400px;
     width: 825px;
    }
    div.widget-area {
     background-color: #ffffff;
     background: #ffffff;
     color: #000000;
     page-break-inside: avoid;
     -moz-box-align: stretch;
     box-orient: horizontal;
     box-align: stretch;
     flex-direction: row;
     align-items: center;
     align-self: auto;
    }
    audio,
    video {
     display: inline;
     vertical-align: middle;
     align-content: center;
     margin-left: 20%;
    }
    div.output.output_scroll {
     box-shadow: none;
    }
    ::-webkit-scrollbar {
     width: 10px;
     background-color: rgba(180,180,180,.25);
     background: rgba(180,180,180,.25);
    }
    ::-webkit-scrollbar-thumb {
     -webkit-border-radius: 2px;
     border-radius: 2px;
     background-color: #aaaaaa;
     background: #aaaaaa;
    }
    ::-webkit-scrollbar-thumb:window-inactive {
     background-color: #dedede;
     background: #dedede;
    }
    span.ansiblack {
     color: #333333;
    }
    span.ansiblue {
     color: #1e70c7;
    }
    span.ansigray {
     color: #656565;
    }
    span.ansigreen {
     color: #009e07;
    }
    span.ansipurple {
     color: #653bc5;
    }
    span.ansicyan {
     color: #009fb7;
    }
    span.ansiyellow {
     color: #dc4384;
    }
    span.ansired {
     color: #de143d;
    }
    div.output-stderr {
     background-color: #ebb5b7;
    }
    div.output-stderr pre {
     color: #000000;
    }
    div.js-error {
     color: #de143d;
    }
    .cm-s-ipython.CodeMirror {
     background: #eeeeee;
     color: #303030;
     border-radius: 2px;
    }
    .cm-s-ipython div.CodeMirror-selected {
     background: #e0e1e3;
    }
    .cm-s-ipython .CodeMirror-gutters {
     background: #e0e1e3;
     border: none;
     border-radius: 0px;
    }
    .cm-s-ipython .CodeMirror-linenumber {
     color: #aaaaaa;
    }
    .cm-s-ipython .CodeMirror-cursor {
     border-left: 2px solid #ff711a;
    }
    .cm-s-ipython span.cm-comment {
     color: #8d8d8d;
     font-style: italic;
    }
    .cm-s-ipython span.cm-atom {
     color: #055be0;
    }
    .cm-s-ipython span.cm-number {
     color: #ff8132;
    }
    .cm-s-ipython span.cm-property {
     color: #e22978;
    }
    .cm-s-ipython span.cm-attribute {
     color: #de143d;
    }
    .cm-s-ipython span.cm-keyword {
     color: #713bc5;
     font-weight: normal;
    }
    .cm-s-ipython span.cm-string {
     color: #009e07;
    }
    .cm-s-ipython span.cm-meta {
     color: #aa22ff;
    }
    .cm-s-ipython span.cm-operator {
     color: #055be0;
    }
    .cm-s-ipython span.cm-builtin {
     color: #e22978;
    }
    .cm-s-ipython span.cm-variable {
     color: #303030;
    }
    .cm-s-ipython span.cm-variable-2 {
     color: #de143d;
    }
    .cm-s-ipython span.cm-variable-3 {
     color: #aa22ff;
    }
    .cm-s-ipython span.cm-def {
     color: #e22978;
     font-weight: normal;
    }
    .cm-s-ipython span.cm-error {
     background: rgba(191,97,106,.40);
    }
    .cm-s-ipython span.cm-tag {
     color: #e22978;
    }
    .cm-s-ipython span.cm-link {
     color: #ff8132;
    }
    .cm-s-ipython span.cm-storage {
     color: #055be0;
    }
    .cm-s-ipython span.cm-entity {
     color: #e22978;
    }
    div.CodeMirror,
    div.CodeMirror pre {
     line-height: 1.6 !important;
     color: #303030;
    }
    div.CodeMirror-lines {
     padding-bottom: .2em;
    }
    div.CodeMirror span.CodeMirror-matchingbracket {
     color: #000000;
     background-color: rgba(30,112,199,.30);
     height: 120%;
     line-height: 1.6 !important;
     padding-top: 1px;
    }
    div.CodeMirror span.CodeMirror-nonmatchingbracket {
     color: #000000;
     background: rgba(191,97,106,.40) !important;
     height: 120%;
     line-height: 1.6 !important;
     padding-top: 1px;
    }
    div.cell.text_cell .cm-s-default .cm-header {
     color: #1e70c7;
    }
    div.cell.text_cell .cm-s-default span.cm-variable-2 {
     color: #4c4c4c;
    }
    div.cell.text_cell .cm-s-default span.cm-variable-3 {
     color: #aa22ff;
    }
    .cm-s-default span.cm-comment {
     color: #8d8d8d;
    }
    .cm-s-default .cm-tag {
     color: #009fb7;
    }
    div.text_cell_render h1 {
     font-family: 'Open-Sans', sans-serif;
     text-align: center;
     font-weight: lighter;
     background: #ffffff;
     background-color: #ffffff;
     color: #1e70c7;
    }
    div.text_cell_render h2,
    div.text_cell_render h3,
    div.text_cell_render h4,
    div.text_cell_render h5,
    div.text_cell_render h6 {
     font-family: 'Open-Sans', sans-serif;
     text-align: left;
     font-weight: lighter;
     background: #ffffff;
     background-color: #ffffff;
     color: #1e70c7;
    }
    body {
     font-family: "Open-Sans", sans-serif;
     font-size: 12pt;
     line-height: 1.45;
     color: #2c2c2c;
     background-color: #ffffff;
    }
    a {
     color: #2c2c2c;
     font-family: "Open-Sans", sans-serif;
    }
    div#notebook,
    div.output_area pre,
    div.output_wrapper,
    div.prompt,
    div.CodeMirror,
    div.CodeMirror pre {
     font-family: "Hack", monospace;
     font-size: 11pt;
    }
    div.cell {
     padding: 0px;
     background: #eeeeee;
     background-color: #eeeeee;
     border: medium solid #ffffff;
     align: center;
     border-top-right-radius: 2px;
     border-top-left-radius: 2px;
    }
    div.input_area {
     background-color: #eeeeee;
     margin-bottom: 0px;
     border: 0px;
     border-top-left-radius: 0px;
     border-top-right-radius: 2px;
     border-bottom-left-radius: 0px;
     border-bottom-right-radius: 0px;
    }
    div.cell.selected {
     background: #eeeeee;
     background-color: #eeeeee;
     border: medium solid #ff8132;
     padding: 0px;
     border-top-right-radius: 2px;
     border-top-left-radius: 2px;
    }
    .edit_mode div.cell.selected {
     background: #eeeeee;
     background-color: #eeeeee;
     border: medium solid #ffd5bb;
     padding: 0px;
     border-top-right-radius: 2px;
     border-top-left-radius: 2px;
    }
    div.cell.edit_mode {
     background: #eeeeee;
     background-color: #eeeeee;
     border: medium solid #ffd5bb;
     padding: 0px;
     border-top-right-radius: 2px;
     border-top-left-radius: 2px;
    }
    div.output_wrapper {
     background-color: #ffffff;
     margin-bottom: 0em;
     margin-top: 0em;
     border: 0px;
     border-top-right-radius: 0px;
     border-top-left-radius: 0px;
     border-bottom-left-radius: 2px;
     border-bottom-right-radius: 2px;
    }
    div.output_subarea.output_text.output_stream.output_stdout,
    div.output_subarea.output_text {
     background-color: #ffffff;
     color: #000000;
     border-top-right-radius: 0px;
     border-top-left-radius: 0px;
     border-bottom-left-radius: 2px;
     border-bottom-right-radius: 2px;
    }
    div.output_area pre {
     color: #000000;
     font-size: 9.5pt;
     border-top-right-radius: 0px;
     border-top-left-radius: 0px;
     border-bottom-left-radius: 2px;
     border-bottom-right-radius: 2px;
    }
    div.output_subarea {
     overflow-x: auto;
     padding: 1.0em;
     -webkit-box-flex: 1;
     -moz-box-flex: 1;
     box-flex: 1;
     flex: 1;
     max-width: calc(86%);
    }
    div.text_cell,
    div.text_cell_render pre,
    div.text_cell_render {
     font-family: "Open-Sans", sans-serif;
     font-size: 100%;
     line-height: 130%;
     margin-right: 0px;
     margin-left: 0px;
     background: #ffffff;
     background-color: #ffffff;
     color: #4c4c4c;
     border-radius: 2px;
    }
    div.cell.text_cell.rendered.selected {
     border: medium solid #1e70c7;
     border-radius: 2px;
    }
    div.cell.text_cell.unrendered.selected {
     border: medium solid #1e70c7;
     background: #ffffff;
     background-color: #ffffff;
     border-radius: 2px;
    }
    div.cell.text_cell.selected {
     border: medium solid #1e70c7;
     background: #ffffff;
     background-color: #ffffff;
     border-radius: 2px;
    }
    .edit_mode div.cell.text_cell.selected {
     background: #ffffff;
     background-color: #ffffff;
     border: medium solid #87b0db;
     border-radius: 2px;
    }
    div.text_cell.unrendered,
    div.text_cell.unrendered.selecteddiv.edit_mode div.text_cell.unrendered {
     background: #ffffff;
     background-color: #ffffff;
     border-radius: 2px;
    }
    .rendered_html table,
    .rendered_html th,
    .rendered_html tr,
    .rendered_html td {
     font-family: "Hack", monospace;
     border: 1px #4c4c4c solid;
     color: #4c4c4c;
     font-size: 9pt;
     background-color: #ffffff;
     background: #ffffff;
     align-content: left !important;
    }
    .rendered-html pre,
    .rendered-html code {
     font-size: 12pt;
     color: #2c2c2c;
     line-height: 100%;
     background: #eeeeee;
    }
    div.output_html {
     color: #4c4c4c;
    }
    div.prompt {
     padding: 0.3em;
     margin: 0px;
     font-family: "Hack", monospace;
     text-align: left;
     line-height: 1.21429em;
     min-width: 9.5ex;
     font-size: 9pt;
    }
    div.prompt.input_prompt {
     font-family: "Hack", monospace;
     color: #aaaaaa;
     background-color: #eeeeee;
     font-weight: normal;
     font-size: 9.5pt;
     width: 1em;
     border-right: 2px solid rgba(240,147,43,.40);
     border-top-left-radius: 2px;
     border-top-right-radius: 0px;
     border-bottom-left-radius: 0px;
     border-bottom-right-radius: 0px;
    }
    div.cell.text_cell.rendered .input_prompt {
     border-right: 2px solid #ffffff;
     min-width: 0ex;
     background: #ffffff;
     background-color: #ffffff;
    }
    div.cell.text_cell.unrendered .input_prompt {
     border-right: 2px solid #ffffff;
     min-width: 0ex;
     background: #ffffff;
     background-color: #ffffff;
    }
    div.prompt.output_prompt {
     font-family: "Hack", monospace;
     background-color: #ffffff;
     color: #ffffff;
     font-size: 9.5pt;
     border-bottom-left-radius: 2px;
     border-top-right-radius: 0px;
     border-top-left-radius: 0px;
     border-bottom-right-radius: 0px;
    }
    div.out_prompt_overlay.prompt {
     font-family: "Hack", monospace;
     font-size: 9.5pt;
     width: 1em;
     background-color: #ffffff;
     border-bottom-left-radius: 2px;
     border-top-right-radius: 0px;
     border-top-left-radius: 0px;
     border-bottom-right-radius: 0px;
    }
    div.out_prompt_overlay.prompt:hover {
     font-family: "Hack", monospace;
     font-size: 9.5pt;
     background-color: #ffffff;
     box-shadow: #e8e8e8 1px 1px 3px 3px inset;
     border-bottom-left-radius: 2px;
     -webkit-border-: 2px;
     -moz-border-radius: 2px;
     border-top-right-radius: 0px;
     border-top-left-radius: 0px;
    }
</style>
